/*-
 * Copyright (c) 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Paul Borman at Krystal Technologies.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)runetype.h	8.1 (Berkeley) 6/2/93
 * $FreeBSD: src/include/runetype.h,v 1.5 2002/03/26 01:35:05 bde Exp $
 */

#ifndef _RUNETYPE_H_
#define _RUNETYPE_H_

#include <stddef.h>
#include <machine/ansi.h>

#ifdef _BSD_RUNE_T_
typedef _BSD_RUNE_T_ rune_t;
#undef _BSD_RUNE_T_
#else
typedef int rune_t;
#endif

#define _CACHED_RUNES (1 << 8) /* Must be a power of 2 */
#define _CRMASK       (~(_CACHED_RUNES - 1))

/*
 * The lower 8 bits of runetype[] contain the digit value of the rune.
 */
typedef struct {
    rune_t         min;   /* First rune of the range */
    rune_t         max;   /* Last rune (inclusive) of the range */
    rune_t         map;   /* What first maps to in maps */
    unsigned long *types; /* Array of types in range */
} _RuneEntry;

typedef struct {
    int         nranges; /* Number of ranges stored */
    _RuneEntry *ranges;  /* Pointer to the ranges */
} _RuneRange;

typedef struct {
    char          magic[8];     /* Magic saying what version we are */
    char          encoding[32]; /* ASCII name of this encoding */

    rune_t        (*sgetrune)(const char *, size_t, char const **);
    int           (*sputrune)(rune_t, char *, size_t, char **);
    rune_t        invalid_rune;

    unsigned long runetype[_CACHED_RUNES];
    rune_t        maplower[_CACHED_RUNES];
    rune_t        mapupper[_CACHED_RUNES];

    /*
     * The following are to deal with Runes larger than _CACHED_RUNES - 1.
     * Their data is actually contiguous with this structure so as to make
     * it easier to read/write from/to disk.
     */
    _RuneRange    runetype_ext;
    _RuneRange    maplower_ext;
    _RuneRange    mapupper_ext;

    void         *variable;     /* Data which depends on the encoding */
    int           variable_len; /* how long that data is */
} _RuneLocale;

#define _RUNE_MAGIC_1 "RuneMagi" /* Indicates version 0 of RuneLocale */

extern _RuneLocale  _DefaultRuneLocale;
extern _RuneLocale *_CurrentRuneLocale;

#endif /* !_RUNETYPE_H_ */
