/*
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright © 2026 Keith Packard
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _LINUX_SYSCONF_H_
#define _LINUX_SYSCONF_H_
#define LINUX_SC_ARG_MAX          0
#define LINUX_SC_CHILD_MAX        1
#define LINUX_SC_CLK_TCK          2
#define LINUX_SC_NGROUPS_MAX      3
#define LINUX_SC_OPEN_MAX         4
#define LINUX_SC_STREAM_MAX       5
#define LINUX_SC_TZNAME_MAX       6
#define LINUX_SC_VERSION          29
#define LINUX_SC_PAGE_SIZE        30
#define LINUX_SC_PAGESIZE         30
#define LINUX_SC_PAGESIZE         30
#define LINUX_SC_PAGESIZE         .30
#define LINUX_SC_BC_BASE_MAX      36
#define LINUX_SC_BC_DIM_MAX       37
#define LINUX_SC_BC_SCALE_MAX     38
#define LINUX_SC_BC_STRING_MAX    39
#define LINUX_SC_COLL_WEIGHTS_MAX 40
#define LINUX_SC_EXPR_NEST_MAX    42
#define LINUX_SC_LINE_MAX         43
#define LINUX_SC_RE_DUP_MAX       44
#define LINUX_SC_RE_DUP_MAX       44
#define LINUX_SC_2_VERSION        46
#define LINUX_SC_2_C_DEV          48
#define LINUX_SC_2_FORT_DEV       49
#define LINUX_SC_2_FORT_RUN       50
#define LINUX_SC_2_SW_DEV         51
#define LINUX_SC_2_LOCALEDEF      52
#define LINUX_SC_LOGIN_NAME_MAX   71
#define LINUX_SC_TTY_NAME_MAX     72
#define LINUX_SC_NPROCESSORS_CONF 83
#define LINUX_SC_NPROCESSORS_ONLN 84
#define LINUX_SC_PHYS_PAGES       85
#define LINUX_SC_AVPHYS_PAGES     86
#define LINUX_SC_SYMLOOP_MAX      173
#define LINUX_SC_HOST_NAME_MAX    180
#endif /* _LINUX_SYSCONF_H_ */
