/*
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright © 2026 Keith Packard
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _LINUX_SYSCALL_H_
#define _LINUX_SYSCALL_H_
#define LINUX_SYS_restart_syscall        0
#define LINUX_SYS_exit                   1
#define LINUX_SYS_fork                   2
#define LINUX_SYS_read                   3
#define LINUX_SYS_write                  4
#define LINUX_SYS_open                   5
#define LINUX_SYS_close                  6
#define LINUX_SYS_waitpid                7
#define LINUX_SYS_creat                  8
#define LINUX_SYS_link                   9
#define LINUX_SYS_unlink                 10
#define LINUX_SYS_execve                 11
#define LINUX_SYS_chdir                  12
#define LINUX_SYS_time                   13
#define LINUX_SYS_mknod                  14
#define LINUX_SYS_chmod                  15
#define LINUX_SYS_lchown                 16
#define LINUX_SYS_oldstat                18
#define LINUX_SYS_lseek                  19
#define LINUX_SYS_getpid                 20
#define LINUX_SYS_mount                  21
#define LINUX_SYS_umount                 22
#define LINUX_SYS_setuid                 23
#define LINUX_SYS_getuid                 24
#define LINUX_SYS_stime                  25
#define LINUX_SYS_ptrace                 26
#define LINUX_SYS_alarm                  27
#define LINUX_SYS_oldfstat               28
#define LINUX_SYS_pause                  29
#define LINUX_SYS_utime                  30
#define LINUX_SYS_access                 33
#define LINUX_SYS_nice                   34
#define LINUX_SYS_sync                   36
#define LINUX_SYS_kill                   37
#define LINUX_SYS_rename                 38
#define LINUX_SYS_mkdir                  39
#define LINUX_SYS_rmdir                  40
#define LINUX_SYS_dup                    41
#define LINUX_SYS_pipe                   42
#define LINUX_SYS_times                  43
#define LINUX_SYS_brk                    45
#define LINUX_SYS_setgid                 46
#define LINUX_SYS_getgid                 47
#define LINUX_SYS_signal                 48
#define LINUX_SYS_geteuid                49
#define LINUX_SYS_getegid                50
#define LINUX_SYS_acct                   51
#define LINUX_SYS_umount2                52
#define LINUX_SYS_ioctl                  54
#define LINUX_SYS_fcntl                  55
#define LINUX_SYS_setpgid                57
#define LINUX_SYS_oldolduname            59
#define LINUX_SYS_umask                  60
#define LINUX_SYS_chroot                 61
#define LINUX_SYS_ustat                  62
#define LINUX_SYS_dup2                   63
#define LINUX_SYS_getppid                64
#define LINUX_SYS_getpgrp                65
#define LINUX_SYS_setsid                 66
#define LINUX_SYS_sigaction              67
#define LINUX_SYS_sgetmask               68
#define LINUX_SYS_ssetmask               69
#define LINUX_SYS_setreuid               70
#define LINUX_SYS_setregid               71
#define LINUX_SYS_sigsuspend             72
#define LINUX_SYS_sigpending             73
#define LINUX_SYS_sethostname            74
#define LINUX_SYS_setrlimit              75
#define LINUX_SYS_getrlimit              76
#define LINUX_SYS_getrusage              77
#define LINUX_SYS_gettimeofday           78
#define LINUX_SYS_settimeofday           79
#define LINUX_SYS_getgroups              80
#define LINUX_SYS_setgroups              81
#define LINUX_SYS_select                 82
#define LINUX_SYS_symlink                83
#define LINUX_SYS_oldlstat               84
#define LINUX_SYS_readlink               85
#define LINUX_SYS_uselib                 86
#define LINUX_SYS_swapon                 87
#define LINUX_SYS_reboot                 88
#define LINUX_SYS_readdir                89
#define LINUX_SYS_mmap                   90
#define LINUX_SYS_munmap                 91
#define LINUX_SYS_truncate               92
#define LINUX_SYS_ftruncate              93
#define LINUX_SYS_fchmod                 94
#define LINUX_SYS_fchown                 95
#define LINUX_SYS_getpriority            96
#define LINUX_SYS_setpriority            97
#define LINUX_SYS_statfs                 99
#define LINUX_SYS_fstatfs                100
#define LINUX_SYS_ioperm                 101
#define LINUX_SYS_socketcall             102
#define LINUX_SYS_syslog                 103
#define LINUX_SYS_setitimer              104
#define LINUX_SYS_getitimer              105
#define LINUX_SYS_stat                   106
#define LINUX_SYS_lstat                  107
#define LINUX_SYS_fstat                  108
#define LINUX_SYS_olduname               109
#define LINUX_SYS_iopl                   110
#define LINUX_SYS_vhangup                111
#define LINUX_SYS_vm86old                113
#define LINUX_SYS_wait4                  114
#define LINUX_SYS_swapoff                115
#define LINUX_SYS_sysinfo                116
#define LINUX_SYS_ipc                    117
#define LINUX_SYS_fsync                  118
#define LINUX_SYS_sigreturn              119
#define LINUX_SYS_clone                  120
#define LINUX_SYS_setdomainname          121
#define LINUX_SYS_uname                  122
#define LINUX_SYS_modify_ldt             123
#define LINUX_SYS_adjtimex               124
#define LINUX_SYS_mprotect               125
#define LINUX_SYS_sigprocmask            126
#define LINUX_SYS_create_module          127
#define LINUX_SYS_init_module            128
#define LINUX_SYS_delete_module          129
#define LINUX_SYS_get_kernel_syms        130
#define LINUX_SYS_quotactl               131
#define LINUX_SYS_getpgid                132
#define LINUX_SYS_fchdir                 133
#define LINUX_SYS_bdflush                134
#define LINUX_SYS_sysfs                  135
#define LINUX_SYS_personality            136
#define LINUX_SYS_setfsuid               138
#define LINUX_SYS_setfsgid               139
#define LINUX_SYS__llseek                140
#define LINUX_SYS_getdents               141
#define LINUX_SYS__newselect             142
#define LINUX_SYS_flock                  143
#define LINUX_SYS_msync                  144
#define LINUX_SYS_readv                  145
#define LINUX_SYS_writev                 146
#define LINUX_SYS_getsid                 147
#define LINUX_SYS_fdatasync              148
#define LINUX_SYS__sysctl                149
#define LINUX_SYS_mlock                  150
#define LINUX_SYS_munlock                151
#define LINUX_SYS_mlockall               152
#define LINUX_SYS_munlockall             153
#define LINUX_SYS_sched_setparam         154
#define LINUX_SYS_sched_getparam         155
#define LINUX_SYS_sched_setscheduler     156
#define LINUX_SYS_sched_getscheduler     157
#define LINUX_SYS_sched_yield            158
#define LINUX_SYS_sched_get_priority_max 159
#define LINUX_SYS_sched_get_priority_min 160
#define LINUX_SYS_sched_rr_get_interval  161
#define LINUX_SYS_nanosleep              162
#define LINUX_SYS_mremap                 163
#define LINUX_SYS_setresuid              164
#define LINUX_SYS_getresuid              165
#define LINUX_SYS_vm86                   166
#define LINUX_SYS_query_module           167
#define LINUX_SYS_poll                   168
#define LINUX_SYS_nfsservctl             169
#define LINUX_SYS_setresgid              170
#define LINUX_SYS_getresgid              171
#define LINUX_SYS_prctl                  172
#define LINUX_SYS_rt_sigreturn           173
#define LINUX_SYS_rt_sigaction           174
#define LINUX_SYS_rt_sigprocmask         175
#define LINUX_SYS_rt_sigpending          176
#define LINUX_SYS_rt_sigtimedwait        177
#define LINUX_SYS_rt_sigqueueinfo        178
#define LINUX_SYS_rt_sigsuspend          179
#define LINUX_SYS_pread64                180
#define LINUX_SYS_pwrite64               181
#define LINUX_SYS_chown                  182
#define LINUX_SYS_getcwd                 183
#define LINUX_SYS_capget                 184
#define LINUX_SYS_capset                 185
#define LINUX_SYS_sigaltstack            186
#define LINUX_SYS_sendfile               187
#define LINUX_SYS_vfork                  190
#define LINUX_SYS_ugetrlimit             191
#define LINUX_SYS_mmap2                  192
#define LINUX_SYS_truncate64             193
#define LINUX_SYS_ftruncate64            194
#define LINUX_SYS_stat64                 195
#define LINUX_SYS_lstat64                196
#define LINUX_SYS_fstat64                197
#define LINUX_SYS_lchown32               198
#define LINUX_SYS_getuid32               199
#define LINUX_SYS_getgid32               200
#define LINUX_SYS_geteuid32              201
#define LINUX_SYS_getegid32              202
#define LINUX_SYS_setreuid32             203
#define LINUX_SYS_setregid32             204
#define LINUX_SYS_getgroups32            205
#define LINUX_SYS_setgroups32            206
#define LINUX_SYS_fchown32               207
#define LINUX_SYS_setresuid32            208
#define LINUX_SYS_getresuid32            209
#define LINUX_SYS_setresgid32            210
#define LINUX_SYS_getresgid32            211
#define LINUX_SYS_chown32                212
#define LINUX_SYS_setuid32               213
#define LINUX_SYS_setgid32               214
#define LINUX_SYS_setfsuid32             215
#define LINUX_SYS_setfsgid32             216
#define LINUX_SYS_pivot_root             217
#define LINUX_SYS_mincore                218
#define LINUX_SYS_madvise                219
#define LINUX_SYS_getdents64             220
#define LINUX_SYS_fcntl64                221
#define LINUX_SYS_gettid                 224
#define LINUX_SYS_readahead              225
#define LINUX_SYS_setxattr               226
#define LINUX_SYS_lsetxattr              227
#define LINUX_SYS_fsetxattr              228
#define LINUX_SYS_getxattr               229
#define LINUX_SYS_lgetxattr              230
#define LINUX_SYS_fgetxattr              231
#define LINUX_SYS_listxattr              232
#define LINUX_SYS_llistxattr             233
#define LINUX_SYS_flistxattr             234
#define LINUX_SYS_removexattr            235
#define LINUX_SYS_lremovexattr           236
#define LINUX_SYS_fremovexattr           237
#define LINUX_SYS_tkill                  238
#define LINUX_SYS_sendfile64             239
#define LINUX_SYS_futex                  240
#define LINUX_SYS_sched_setaffinity      241
#define LINUX_SYS_sched_getaffinity      242
#define LINUX_SYS_set_thread_area        243
#define LINUX_SYS_get_thread_area        244
#define LINUX_SYS_io_setup               245
#define LINUX_SYS_io_destroy             246
#define LINUX_SYS_io_getevents           247
#define LINUX_SYS_io_submit              248
#define LINUX_SYS_io_cancel              249
#define LINUX_SYS_fadvise64              250
#define LINUX_SYS_exit_group             252
#define LINUX_SYS_lookup_dcookie         253
#define LINUX_SYS_epoll_create           254
#define LINUX_SYS_epoll_ctl              255
#define LINUX_SYS_epoll_wait             256
#define LINUX_SYS_remap_file_pages       257
#define LINUX_SYS_set_tid_address        258
#define LINUX_SYS_timer_create           259
#define LINUX_SYS_timer_settime          260
#define LINUX_SYS_timer_gettime          261
#define LINUX_SYS_timer_getoverrun       262
#define LINUX_SYS_timer_delete           263
#define LINUX_SYS_clock_settime          264
#define LINUX_SYS_clock_gettime          265
#define LINUX_SYS_clock_getres           266
#define LINUX_SYS_clock_nanosleep        267
#define LINUX_SYS_statfs64               268
#define LINUX_SYS_fstatfs64              269
#define LINUX_SYS_tgkill                 270
#define LINUX_SYS_utimes                 271
#define LINUX_SYS_fadvise64_64           272
#define LINUX_SYS_mbind                  274
#define LINUX_SYS_get_mempolicy          275
#define LINUX_SYS_set_mempolicy          276
#define LINUX_SYS_mq_open                277
#define LINUX_SYS_mq_unlink              278
#define LINUX_SYS_mq_timedsend           279
#define LINUX_SYS_mq_timedreceive        280
#define LINUX_SYS_mq_notify              281
#define LINUX_SYS_mq_getsetattr          282
#define LINUX_SYS_kexec_load             283
#define LINUX_SYS_waitid                 284
#define LINUX_SYS_add_key                286
#define LINUX_SYS_request_key            287
#define LINUX_SYS_keyctl                 288
#define LINUX_SYS_ioprio_set             289
#define LINUX_SYS_ioprio_get             290
#define LINUX_SYS_inotify_init           291
#define LINUX_SYS_inotify_add_watch      292
#define LINUX_SYS_inotify_rm_watch       293
#define LINUX_SYS_migrate_pages          294
#define LINUX_SYS_openat                 295
#define LINUX_SYS_mkdirat                296
#define LINUX_SYS_mknodat                297
#define LINUX_SYS_fchownat               298
#define LINUX_SYS_futimesat              299
#define LINUX_SYS_fstatat64              300
#define LINUX_SYS_unlinkat               301
#define LINUX_SYS_renameat               302
#define LINUX_SYS_linkat                 303
#define LINUX_SYS_symlinkat              304
#define LINUX_SYS_readlinkat             305
#define LINUX_SYS_fchmodat               306
#define LINUX_SYS_faccessat              307
#define LINUX_SYS_pselect6               308
#define LINUX_SYS_ppoll                  309
#define LINUX_SYS_unshare                310
#define LINUX_SYS_set_robust_list        311
#define LINUX_SYS_get_robust_list        312
#define LINUX_SYS_splice                 313
#define LINUX_SYS_sync_file_range        314
#define LINUX_SYS_tee                    315
#define LINUX_SYS_vmsplice               316
#define LINUX_SYS_move_pages             317
#define LINUX_SYS_getcpu                 318
#define LINUX_SYS_epoll_pwait            319
#define LINUX_SYS_utimensat              320
#define LINUX_SYS_signalfd               321
#define LINUX_SYS_timerfd_create         322
#define LINUX_SYS_eventfd                323
#define LINUX_SYS_fallocate              324
#define LINUX_SYS_timerfd_settime        325
#define LINUX_SYS_timerfd_gettime        326
#define LINUX_SYS_signalfd4              327
#define LINUX_SYS_eventfd2               328
#define LINUX_SYS_epoll_create1          329
#define LINUX_SYS_dup3                   330
#define LINUX_SYS_pipe2                  331
#define LINUX_SYS_inotify_init1          332
#define LINUX_SYS_preadv                 333
#define LINUX_SYS_pwritev                334
#define LINUX_SYS_rt_tgsigqueueinfo      335
#define LINUX_SYS_perf_event_open        336
#define LINUX_SYS_recvmmsg               337
#define LINUX_SYS_fanotify_init          338
#define LINUX_SYS_fanotify_mark          339
#define LINUX_SYS_prlimit64              340
#define LINUX_SYS_name_to_handle_at      341
#define LINUX_SYS_open_by_handle_at      342
#define LINUX_SYS_clock_adjtime          343
#define LINUX_SYS_syncfs                 344
#define LINUX_SYS_sendmmsg               345
#define LINUX_SYS_setns                  346
#define LINUX_SYS_process_vm_readv       347
#define LINUX_SYS_process_vm_writev      348
#define LINUX_SYS_kcmp                   349
#define LINUX_SYS_finit_module           350
#define LINUX_SYS_sched_setattr          351
#define LINUX_SYS_sched_getattr          352
#define LINUX_SYS_renameat2              353
#define LINUX_SYS_seccomp                354
#define LINUX_SYS_getrandom              355
#define LINUX_SYS_memfd_create           356
#define LINUX_SYS_bpf                    357
#define LINUX_SYS_execveat               358
#define LINUX_SYS_socket                 359
#define LINUX_SYS_socketpair             360
#define LINUX_SYS_bind                   361
#define LINUX_SYS_connect                362
#define LINUX_SYS_listen                 363
#define LINUX_SYS_accept4                364
#define LINUX_SYS_getsockopt             365
#define LINUX_SYS_setsockopt             366
#define LINUX_SYS_getsockname            367
#define LINUX_SYS_getpeername            368
#define LINUX_SYS_sendto                 369
#define LINUX_SYS_sendmsg                370
#define LINUX_SYS_recvfrom               371
#define LINUX_SYS_recvmsg                372
#define LINUX_SYS_shutdown               373
#define LINUX_SYS_userfaultfd            374
#define LINUX_SYS_membarrier             375
#define LINUX_SYS_mlock2                 376
#define LINUX_SYS_copy_file_range        377
#define LINUX_SYS_preadv2                378
#define LINUX_SYS_pwritev2               379
#define LINUX_SYS_pkey_mprotect          380
#define LINUX_SYS_pkey_alloc             381
#define LINUX_SYS_pkey_free              382
#define LINUX_SYS_statx                  383
#define LINUX_SYS_arch_prctl             384
#define LINUX_SYS_io_pgetevents          385
#define LINUX_SYS_rseq                   386
#define LINUX_SYS_semget                 393
#define LINUX_SYS_semctl                 394
#define LINUX_SYS_shmget                 395
#define LINUX_SYS_shmctl                 396
#define LINUX_SYS_shmat                  397
#define LINUX_SYS_shmdt                  398
#define LINUX_SYS_msgget                 399
#define LINUX_SYS_msgsnd                 400
#define LINUX_SYS_msgrcv                 401
#define LINUX_SYS_msgctl                 402
#define LINUX_SYS_pidfd_send_signal      424
#define LINUX_SYS_io_uring_setup         425
#define LINUX_SYS_io_uring_enter         426
#define LINUX_SYS_io_uring_register      427
#define LINUX_SYS_open_tree              428
#define LINUX_SYS_move_mount             429
#define LINUX_SYS_fsopen                 430
#define LINUX_SYS_fsconfig               431
#define LINUX_SYS_fsmount                432
#define LINUX_SYS_fspick                 433
#define LINUX_SYS_pidfd_open             434
#define LINUX_SYS_clone3                 435
#define LINUX_SYS_close_range            436
#define LINUX_SYS_openat2                437
#define LINUX_SYS_pidfd_getfd            438
#define LINUX_SYS_faccessat2             439
#define LINUX_SYS_process_madvise        440
#define LINUX_SYS_epoll_pwait2           441
#define LINUX_SYS_quotactl_fd            443
#define LINUX_SYS_landlock_create_rulese 444
#define LINUX_SYS_landlock_add_rule      445
#define LINUX_SYS_landlock_restrict_self 446
#define LINUX_SYS_memfd_secret           447
#endif /* _LINUX_SYSCALL_H_ */
