/* Copyright (c) 2016 Corinna Vinschen <corinna@vinschen.de> */
/* Definition of opaque POSIX-1.2008 type locale_t for userspace. */

#ifndef _SYS__LOCALE_H
#define _SYS__LOCALE_H

typedef int locale_t;

#endif /* _SYS__LOCALE_H */
