/* Copyright (c) 2003 Kazu Hirata  <kazu@cs.umass.edu> */
#ifdef __H8300H__
#ifdef __NORMAL_MODE__
.h8300hn
#else
.h8300h
#endif
#endif
#ifdef __H8300S__
#ifdef __NORMAL_MODE__
    .h8300sn
#else
    .h8300s
#endif
#endif
#ifdef __H8300SX__
#ifdef __NORMAL_MODE__
    .h8300sxn
#else
    .h8300sx
#endif
#endif
