/*
Copyright (c) 1994 Cygnus Support.
All rights reserved.

Redistribution and use in source and binary forms are permitted
provided that the above copyright notice and this paragraph are
duplicated in all such forms and that any documentation,
and/or other materials related to such
distribution and use acknowledge that the software was developed
at Cygnus Support, Inc.  Cygnus Support, Inc. may not be used to
endorse or promote products derived from this software without
specific prior written permission.
THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
/*
FUNCTION
        <<strcpy>>---copy string

INDEX
        strcpy

SYNOPSIS
        #include <string.h>
        char *strcpy(char *<[dst]>, const char *<[src]>);

DESCRIPTION
        <<strcpy>> copies the string pointed to by <[src]>
        (including the terminating null character) to the array
        pointed to by <[dst]>.

RETURNS
        This function returns the initial value of <[dst]>.

PORTABILITY
<<strcpy>> is ANSI C.

<<strcpy>> requires no supporting OS subroutines.

QUICKREF
        strcpy ansi pure
*/

#include <string.h>
#include <limits.h>
#include "local.h"

/*SUPPRESS 560*/
/*SUPPRESS 530*/

#undef strcpy

char *
strcpy(char *dst0, const char *src0)
{
#if defined(__PREFER_SIZE_OVER_SPEED) || defined(__OPTIMIZE_SIZE__) \
    || defined(_PICOLIBC_NO_OUT_OF_BOUNDS_READS)
    char *s = dst0;

    while ((*dst0++ = *src0++))
        ;

    return s;
#else
    char       *dst = dst0;
    const char *src = src0;
    long       *aligned_dst;
    const long *aligned_src;

    /* If SRC or DEST is unaligned, then copy bytes.  */
    if (!UNALIGNED_X_Y(src, dst)) {
        aligned_dst = (long *)dst;
        aligned_src = (long *)src;

        /* SRC and DEST are both "long int" aligned, try to do "long int"
           sized copies.  */
        while (!DETECT_NULL(*aligned_src)) {
            *aligned_dst++ = *aligned_src++;
        }

        dst = (char *)aligned_dst;
        src = (char *)aligned_src;
    }

    while ((*dst++ = *src++))
        ;
    return dst0;
#endif /* not __PREFER_SIZE_OVER_SPEED */
}
