/*
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright © 2026 Keith Packard
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define _GNU_SOURCE
#include <unistd.h>
#include <limits.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

int
execvpe(const char *file, char * const argv[], char * const env[])
{
    if (strchr(file, '/'))
        return execve(file, argv, env);

    char        path[PATH_MAX + 1];
    char       *p, *colon;
    size_t      file_len = strlen(file);
    ptrdiff_t   dir_len;
    const char *exec_path;
    int         ret;

    p = getenv("PATH");
    if (!p)
        p = "";
    for (;;) {
        colon = strchrnul(p, ':');
        dir_len = colon - p;
        if (dir_len + file_len + 2 <= sizeof(path)) {
            if (dir_len > 0) {
                memcpy(path, p, dir_len);
                path[dir_len] = '/';
                memcpy(path + dir_len + 1, file, file_len);
                path[dir_len + 1 + file_len] = '\0';
                exec_path = path;
            } else {
                exec_path = file;
            }
            ret = execve(exec_path, argv, env);
            if (errno != ENOENT)
                return ret;
        }
        if (!*colon) {
            errno = ENOENT;
            return -1;
        }
        p = colon + 1;
    }
}
