/*-
 * Copyright (c) 2002 Mike Barcroft <mike@FreeBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD: head/include/strings.h 272673 2014-10-07 04:54:11Z delphij $
 */

#ifndef _STRINGS_H_
#define _STRINGS_H_

#include <sys/cdefs.h>
#include <sys/_types.h>

#if __POSIX_VISIBLE >= 200809
#include <sys/_locale.h>
#endif

// As per the GCC source, _SIZE_T_DECLARD is FreeBSD 5
// So: Also check for _SIZE_T
#if !defined(_SIZE_T_DECLARED) && !defined(_SIZE_T)
typedef __size_t size_t;
#endif

#ifndef _SIZE_T_DECLARED
#define _SIZE_T_DECLARED
#endif

#ifndef _SIZE_T
#define _SIZE_T
#endif

_BEGIN_STD_C
#if __BSD_VISIBLE || __POSIX_VISIBLE <= 200112
int
       bcmp(const void *, const void *, size_t)
__pure __picolibc_export;                                     /* LEGACY */
void   bcopy(const void *, void *, size_t) __picolibc_export; /* LEGACY */
void   bzero(void *, size_t) __picolibc_export;               /* LEGACY */
#endif
#if __BSD_VISIBLE
void explicit_bzero(void *, size_t) __picolibc_export;
#endif
#if __MISC_VISIBLE || __POSIX_VISIBLE < 200809 || __XSI_VISIBLE >= 700
int ffs(int) __const __picolibc_export;
#endif
#if __BSD_VISIBLE
int ffsl(long) __const __picolibc_export;
int ffsll(long long) __const __picolibc_export;
int fls(int) __const __picolibc_export;
int flsl(long) __const __picolibc_export;
int flsll(long long) __const __picolibc_export;
#endif
#if __BSD_VISIBLE || __POSIX_VISIBLE <= 200112
char *index(const char *, int) __pure __picolibc_export;  /* LEGACY */
char *rindex(const char *, int) __pure __picolibc_export; /* LEGACY */
#endif
int strcasecmp(const char *, const char *) __pure __picolibc_export;
int
       strncasecmp(const char *, const char *, size_t)
__pure __picolibc_export;

#if __POSIX_VISIBLE >= 200809
int strcasecmp_l(const char *, const char *, locale_t) __picolibc_export;
int strncasecmp_l(const char *, const char *, size_t, locale_t) __picolibc_export;
#endif
_END_STD_C

#if __SSP_FORTIFY_LEVEL > 0
#include <ssp/strings.h>
#endif

#endif /* _STRINGS_H_ */
